\name{mypbc}
\alias{mypbc}
\non_function{}
\title{my version of the PBC data of the survival package}
\description{ my version of the PBC data of the survival package
}
\source{ survival package 
}
\keyword{datasets}
